INSTANT_ARTICLE = {
    "force_validate": False,
    "feed_details": {
        "route-name.xml": {
            "model": "app.models.User",
            "title": "",
            "description": "",
            "lang": "en-us",
            "brand": "",
            "type": "instant-article",  # feed, instant-article
        }
    },
}



from masonite.configuration import config


class InstantArticle:
    _id = ""
    _title = ""
    _subtitle = ""
    _kicker = ""
    _summary = ""
    _description = ""
    _link = ""
    _cover = ""
    _author = ""
    _updated = ""
    _published = ""

    def __init__(self, data: list = []):
        # get key value for loop
        for key, value in data.items():
            # set key value
            setattr(self, key, value)

    @staticmethod
    def create(data: list = []):
        return InstantArticle(data)

    def id(self, id):
        self._id = id
        return self

    def title(self, title: str):
        self._title = title
        return self

    def subtitle(self, subtitle: str):
        self._subtitle = subtitle
        return self

    def kicker(self, kicker: str):
        self._kicker = kicker
        return self

    def cover(self, cover: str):
        self._cover = cover
        return self

    def published(self, published: str):
        self._published = published
        return self

    def updated(self, updated: str):
        self._updated = updated
        return self

    def description(self, description: str):
        self._description = description
        return self

    def link(self, link: str):
        self._link = link
        return self

    def author(self, author: str):
        self._author = author
        return self

    def validate(self):
        force = (
            config("instant_article")
            .get("instant_article")
            .get("force_validate", False)
        )
        _required_fields = [
            "id",
            "title",
            "updated",
            "summary",
            "description",
            "published",
            "link",
            "author",
        ]
        for field in _required_fields:
            if not self.__get(field):
                if force:
                    raise Exception(f"{field} is required")
                else:
                    return False
        return True

    def __get(self, key):
        if hasattr(self, key):
            return getattr(self, key)
        raise Exception(f"{key} is not a valid key")



from datetime import datetime
from ..interfaces.instant_article_interface import InstantArticleInterface
from .instant_article import InstantArticle
from masonite.response import Response
import importlib


class InstantArticleResolver:
    _title = ""
    _description = ""
    _language = "en-us"
    _url = ""
    _brand = ""
    _view = ""
    _items = ""
    _response = None

    def __init__(self, feed, url, resolver, feed_type, response, view) -> None:
        self._title = feed.get("title", None)
        self._brand = feed.get("brand", None)
        self._description = feed.get("description", None)
        self._language = feed.get("language", None)
        self._url = url
        self._feed_type = feed_type
        self._items = self._resolve_items(resolver)
        self._view = view
        self._response = response

    def to_response(self) -> Response:
        data = {
            "title": self._title,
            "description": self._description,
            "language": self._language,
            "link": self._url,
            "brand": self._brand,
            "view": self._view,
            "articles": self._items,
            "last_updated": self._last_updated(),
            "year": datetime.now().year,
        }

        self._response.header("Content-Type", "application/xml;charset=UTF-8")
        if self._feed_type == 'feed':
            return self._view.render('instant_article.feeds', data)
        return self._view.render('instant_article.instant-article', data)

    def _resolve_items(self, resolver):
        try:
            module = importlib.import_module(resolver)

            model = getattr(module, resolver.split(".")[-1])

            items = []
            items = model.get_feed_items()

            # map through items and cast to feed item
            items = list(map(lambda item: self._cast_to_feed_item(item), items))

            return items

        except Exception as e:
            raise Exception(e)

    def _last_updated(self) -> str:
        if self._items:
            return self._items[0]._updated
        return ""

    def _cast_to_feed_item(self, instant_article) -> InstantArticle:
        if isinstance(instant_article, list):
            instant_article = InstantArticle(instant_article)

        if isinstance(instant_article, InstantArticle):
            instant_article.validate()
            return instant_article

        if not isinstance(instant_article, InstantArticleInterface):
            raise Exception(
                "InstantArticle must be an instance of InstantArticleInterface"
            )

        feed_item = instant_article.format_feed()

        if not isinstance(feed_item, InstantArticle):
            raise Exception("InstantArticle must return an instance of InstantArticle")

        feed_item.validate()
        return feed_item



from masonite.packages import PackageProvider

from ..commands.instant_command import InstantCommand
from ..routes import ROUTES


class InstantArticleProvider(PackageProvider):
    
    def configure(self):
        (
            self.root("instant_article")
            .name("instant_article")
            .commands(InstantCommand())
            .config("config/instant_article.py", publish=True)
            .views("views", publish=True)
            
        )

    def register(self):
        # super().register()
        self.application.make("router").add(ROUTES)
        # self.application.make("commands").add(InstantCommand)
        # current_path = Path(__file__).parent.parent.resolve()
        # self.application.make("view").add_location(os.path.join(current_path, 'views'), loader=FileSystemLoader)

    def boot(self):
        pass



from masonite.routes import Route
from ..controllers.instant_article_controller import InstantArticleController

ROUTES = Route.group(
    [
        Route.get("/iarticle", InstantArticleController.index),
        Route.get("/iarticle/@model", InstantArticleController.feeds),
    ]
)



from masonite.controllers import Controller
from masonite.views import View
from masonite.response import Response
from masonite.request import Request
from masonite.configuration import config

from ..models.instant_article_resolver import InstantArticleResolver


class InstantArticleController(Controller):
    def index(self, response: Response):
        return response.json({"message": "Index Page"})

    def feeds(self, slug, view: View, response: Response, request: Request):
        articles = config("instant_article").get("instant_article").get("feed_details")

        # get keys
        keys = list(articles.keys())

        if slug not in keys:
            return response.json({"message": "Feed not found"})

        feed = articles.get(slug)

        url = "{}://{}".format(
            request.environ.get("wsgi.url_scheme"), request.environ.get("HTTP_HOST")
        )

        return InstantArticleResolver(
            feed=feed,
            url=url,
            resolver=feed.get("model", ""),
            feed_type=feed.get("type", "feeds"),
            response=response,
            view=view,
        ).to_response()
